#include <stdio.h>
#include <stdlib.h> // for random numbers
#include <dev/usartavr.h>

#include <hardware/btn-hardware.h>
#include <hardware/btn-bat.h>

#include <bt/bt_hci_cmds.h>
#include <bt/bt_hci_defs.h>
#include <bt/bt_hci_api.h>
#include <bt/bt_acl_com.h>

#include <terminal/btn-terminal.h>
#include <terminal/btn-cmds.h>
#include <terminal/bt-cmds.h>
#include <terminal/nut-cmds.h>
#include <terminal/acl-com-cmds.h>

#include <led/btn-led.h>
#include <sys/thread.h>
#include <sys/heap.h>

#include <sync/sync.h>
#include <sync/sync-anchor.h>

#include <string.h>

#include <dev/irqreg.h>

/********************************/
/* defines                      */
/********************************/

//#define AUTO_ANSWER
#define AUTO_ANSWER2
#define ANSWER_LEN 10

#define AUTO_COMMUNICATE
//#define AUTOSEND_LEN (BT_MAX_ACL_COM_PAYLOAD - 1)
#define AUTOSEND_LEN 40
#define AUTO_LEN_RANDOM
#define AUTO_COM_INIT_FREQ 	4*60*4
#define AUTO_RANDOM 		2*60*4	// minutes +-
#define AUTO_INQ_NUM 12
#define AUTO_COMMUNICATE_DEBUG 0

#define	GROUP0 "00:04:3f:00:00:7b" // Anchornode
#define GROUP1 "00:04:3f:00:00:59", "00:04:3f:00:00:5d", "00:04:3f:00:00:5f"
#define GROUP2 "00:04:3f:00:00:7c", "00:04:3f:00:00:7d", "00:04:3f:00:00:7f"
#define GROUP3 "00:04:3f:00:00:82", "00:04:3f:00:00:86", "00:04:3f:00:00:3c"

#define NUM_OK_ADDR 4
#define ALLOWED GROUP0 , GROUP1

//#define PUT_BOUNDS
#define	PUT_BOUNDS_FREQ 10*4

#ifndef IS_ANCHORNODE
	#define STORE_BOUNDS_LOCAL
	#define STORE_BOUNDS_LOCAL_SIZE 600
	
	#ifdef STORE_BOUNDS_LOCAL
		#define EXTERNAL_INTERRUPT
	#endif // STORE_BOUNDS_LOCAL
#endif // not IS_ANCHORNODE

#define ESSENTIALS_ONLY

//#define ANCHOR_BEAUTY

/********************************/
/* variables                    */
/********************************/

struct btstack* stack;
bt_acl_com_stack_t* acl_com_stack;

extern u_char _bt_hci_debug_uart;

#ifdef AUTO_COMMUNICATE
	u_char _global_c_ok_addr[NUM_OK_ADDR][18] = {ALLOWED};
#endif

typedef struct _sync_test_globals_struct {
	syncbounds boundmemory;
	bt_addr_t my_bt_addr;
	#ifdef AUTO_ANSWER
		short answer_state;
	#endif
	#ifdef AUTO_COMMUNICATE
		int send_freq;
		bt_addr_t ok_addr[NUM_OK_ADDR];
		struct bt_hci_inquiry_result inqres[AUTO_INQ_NUM];	
	#endif
	#ifdef LLDLY
		long long lldly_send_ll;
		long long lldly_rcvd_ll;
		u_char lldly_tstamp[12];
	#endif
	#ifdef STORE_BOUNDS_LOCAL
	   	syncbounds stored_bounds[STORE_BOUNDS_LOCAL_SIZE];
		int num_stored_bounds;
	#endif
	#ifdef EXTERNAL_INTERRUPT
		u_long sync_measure_time;
		u_char sync_measure_signaled;
	#endif

} _sync_test_globals_t;

_sync_test_globals_t * _sync_test_globals;

#ifdef AUTO_COMMUNICATE
	extern char _btn_led_heatbeat_pause;
#endif

#ifdef LLDLY
	extern u_long lldly_send, lldly_rcvd;
	void _cmd_glldly(u_char *);
#endif



/********************************/
/* functions                    */
/********************************/

//prints out current bounds an their average
void _cmd_print_time(u_char* arg){
	long long av_now = 0;
	syncbounds bounds_rcvd;

	#ifdef IS_ANCHORNODE
		printf("\naahrg - RUN!!!\n");
		NutSleep(4000);
	#endif
	bounds_rcvd = rt_get_bounds();
	printf("\n*** time: lower= ");
	_print_dec(bounds_rcvd.lower);
	printf("; upper= ");
	_print_dec(bounds_rcvd.upper);
	rt_get_average(&av_now);
	printf("; average= ");
	_print_dec(av_now);
	printf("; valid? %d ", rt_is_valid());
	printf("\n");
}

#ifndef ESSENTIALS_ONLY
	
	#ifndef BP_ISA
	//"synchronizes" the rtc's bounds with a (probably) narrower interval
	//	sized 20 millis, centered around the current average
	void _cmd_pseudo_sync(u_char* arg){
		long long av_now = 0;
		
		rt_get_average(&av_now);
		_sync_test_globals->boundmemory.lower = av_now - 10;
		_sync_test_globals->boundmemory.upper = av_now + 10;
		#ifdef SYNC_RTT
			sync_sync_bounds(&_sync_test_globals->boundmemory, RT_NOW, 0);		
		#else // not SYNC_RTT
			sync_sync_bounds(&_sync_test_globals->boundmemory, RT_NOW);		
		#endif // SYNC_RTT
	}
	
		#ifndef SYNC_RTT
			void rt_reset(void){
				sync_bounds_global.lower = 0;
				sync_bounds_global.upper = 0;
				sync_bounds_global.validity = 0;
				rt_init();
			}
			
			void _cmd_reset_rtc(u_char * arg){
				rt_reset();
				printf("*** Real time clock reset.\n");
			}
		#endif // !SYNC_RTT
	
	#endif // !BP_ISA
	
	//takes two long integers (signed) from the minicom and does
	//	intersection and synchronization as it would do with bounds
	//	obtained by the hci functions
	// added support for 1 argument. both bounds are set to the same number
	// TODO: lld numbers can't be entered!!!
	// ->now at least scaled ld numbers can be entered...
	
	void _cmd_sync_manual(u_char* arg){
		long lower, upper, prescale;
		long long scale;
		syncbounds manual_bounds = {0, 0, 0, 1};
	
		#ifdef BP_ISA
			syncbounds tmp_bounds = {0,0,0,0};
		#endif
	
		if(sscanf(arg, "sc%ld%ld%ld", &prescale, &lower, &upper) == 3){
			scale = prescale * 100000;
			manual_bounds.lower = (long long) (lower * scale);
			manual_bounds.upper = (long long) (upper * scale);
		}else if(sscanf(arg, "%ld%ld", &lower, &upper) == 2){
			manual_bounds.lower = (long long) lower;
			manual_bounds.upper = (long long) upper;
		}else if (sscanf(arg, "%ld", &lower) == 1){
			manual_bounds.lower = (long long) lower;
			manual_bounds.upper = (long long) lower;
		}else{
			printf("usage: [sc <hundredthousands>] <lower> <upper>\n");
			return;
		}
		#ifdef BP_ISA
		#ifdef SYNC_RTT
				sync_sync_bounds(&manual_bounds, &tmp_bounds, sync_globals.nearest_neighbour, sync_globals.nearest_neighbour->bounds.last_update, 0);		
			#else // not SYNC_RTT
				sync_sync_bounds(&manual_bounds, &tmp_bounds, sync_globals.nearest_neighbour, sync_globals.nearest_neighbour->bounds.last_update);		
			#endif // SYNC_RTT
		#else // not BP_ISA
			SYNC_CURRENT_BOUNDS.validity = 1;
			#ifdef SYNC_RTT
				sync_sync_bounds(&_sync_test_globals->boundmemory, RT_NOW, 0);		
			#else // not SYNC_RTT
				sync_sync_bounds(&_sync_test_globals->boundmemory, RT_NOW);		
			#endif // SYNC_RTT
		#endif // BP_ISA
	}
	
	
	// sends a packet to a btnode running 'answer' and measures time difference between sending
	// time and receive time of reply.
	void _cmd_start(u_char* arg) {
		int handle, res, nrdata;
		int dif;
		u_long start, end;
		if(sscanf(arg, "%u%u", &handle, &nrdata) == 2){
	      
	        //acl-pkt
		    bt_acl_com_pkt_t _acl_com_pkt;
	    	//cast payload to acl_com
		    u_char *acl_com_data = bt_acl_com_get_data_pointer(&_acl_com_pkt);
	
		    bt_acl_com_pkt_t *p_pkt = NULL;
		    bt_hci_con_handle_t con_handle;
		    
		    int i;
		    u_short len;
		    
		    for (i = 0; i < nrdata; i++) {
	    	   acl_com_data[i] = i;
			}       
			
			start = NutGetRealMillis();
	        res = bt_acl_com_send_packet(acl_com_stack, &_acl_com_pkt, handle, 4, (u_short) nrdata);		
			// printf("Sent packet. Waiting.\n");
			res = bt_acl_com_get_packet(acl_com_stack, (u_char) 4, &p_pkt, &con_handle, &len, 0);		
			// printf("Received answer.\n");
			end = NutGetRealMillis();
			
			dif = (int)(end - start);
			printf("%i#\n", dif);
			//_print_dec(dif);
			//printf("\n");		
		}else{
			printf("usage: start <handle> <length>\n");
		}
	}
	
	void _cmd_dirty_anchor(u_char* arg){
		
		memcpy(sync_globals.sync_timestamp_rcvd, arg, SYNC_TSTAMP_LEN);
		#if defined(BP_ISA) || defined (SYNC_RTT)
			bt_hci_read_bt_addr(stack, BT_HCI_SYNC, _sync_test_globals->my_bt_addr);
			#ifndef SYNC_RTT
				sync_process_timestamp(_sync_test_globals->my_bt_addr, 0);
			#else // SYNC_RTT
				sync_process_timestamp(_sync_test_globals->my_bt_addr);
			#endif // SYNC_RTT
		#else
			sync_process_timestamp(0);
		#endif
	}
	
#endif // ESSENTIALS_ONLY


#ifdef AUTO_ANSWER
// automatically replies to incoming pakets
THREAD(answerthread, arg) {
    //acl-pkt
    bt_acl_com_pkt_t _acl_com_pkt;
	//cast payload to acl_com
    u_char *acl_com_data = bt_acl_com_get_data_pointer(&_acl_com_pkt);

    bt_acl_com_pkt_t *p_pkt = NULL;
    bt_hci_con_handle_t con_handle;
    
    int i;
    u_short len;

    for (i = 0; i < ANSWER_LEN; i++) {
	   acl_com_data[i] = i;
	} 
	NutThreadSetPriority(80);
	printf("Answerthread: Ready.\n");
	while(1) {		
		if(_sync_test_globals->answer_state == 1){	
			//printf("Waiting for paket.\n");
			bt_acl_com_get_packet(acl_com_stack, (u_char) 4, &p_pkt, &con_handle, &len, 0);
			bt_acl_com_free_packet(acl_com_stack, p_pkt);
			//printf("Paket received and digested.\n");
	        bt_acl_com_send_packet(acl_com_stack, &_acl_com_pkt, con_handle, 4, (u_short) ANSWER_LEN);
			//printf("Answerpaket sent.\n");
			#ifdef LLDLY
			_cmd_glldly("");
			#endif
			//NutThreadYield();
		}
        if(_sync_test_globals->answer_state == 0){
			NutSleep(2000);
        }
	}
}

void _cmd_answer(u_char* arg) {
	int arg1;	
	if(sscanf(arg, "%u", &arg1) == 1){       
        _sync_test_globals->answer_state = arg1;		
	}
	else
	{
		printf("usage: answer (0: off | 1:on)\n");	
	} 
}
#endif

#ifdef AUTO_ANSWER2
// automatically replies to incoming pakets
THREAD(answerthread2, arg) {
    //acl-pkt
    bt_acl_com_pkt_t _acl_com_pkt;
	//cast payload to acl_com
    u_char *acl_com_data = bt_acl_com_get_data_pointer(&_acl_com_pkt);

    bt_acl_com_pkt_t *p_pkt = NULL;
    bt_hci_con_handle_t con_handle;
    
    int i;
    u_short len;
    u_char* first_byte;

	// first Byte has to be 1 so that answerthread2 on other BTnode
	// doesn't respond causing "feedback"
	acl_com_data[0] = 1;			
    for (i = 1; i < ANSWER_LEN; i++) {
	   acl_com_data[i] = i;
	} 
	NutThreadSetPriority(80);

	while(1) {		
			//printf("Waiting for paket.\n");
			bt_acl_com_get_packet(acl_com_stack, (u_char) 4, &p_pkt, &con_handle, &len, 0);
			
			first_byte = bt_acl_com_get_data_pointer (p_pkt);
			if(*first_byte == 0) {
				bt_acl_com_send_packet(acl_com_stack, &_acl_com_pkt, con_handle, 4, (u_short) ANSWER_LEN);
				//printf("Answerpaket sent.\n");	
			}
			bt_acl_com_free_packet(acl_com_stack, p_pkt);
			//printf("Paket received and digested.\n");	        
	}
}

#endif

// is called by a pc program to read out the current bounds
// prints timestamp format
void _cmd_gbound(u_char* arg) {
	#ifdef BP_ISA
		//sync_generate_timestamp(sync_globals.nearest_neighbour->address);
		_sync_test_globals->boundmemory = get_updated_bounds(&sync_globals.nearest_neighbour->bounds, RT_NOW + SYNC_COMPUTATIONAL_DELAY);
		sync_assemble_timestamp(&_sync_test_globals->boundmemory, &_sync_test_globals->boundmemory);
	#else
		_sync_test_globals->boundmemory = get_updated_bounds(&sync_globals.global_bounds, RT_NOW + SYNC_COMPUTATIONAL_DELAY);
		sync_assemble_timestamp(&_sync_test_globals->boundmemory);
	#endif
	int i;
	printf("#");
	for(i=0; i < SYNC_TSTAMP_LEN; i += 1){
		printf("%c", ((sync_globals.sync_timestamp_to_send[i] & 0x0f) | 0x50));
		printf("%c", ((sync_globals.sync_timestamp_to_send[i] >> 4) | 0x50));
	}
	printf(ACK_CHAR "\n");
}


#ifdef AUTO_COMMUNICATE

void print_bt_addr(bt_addr_t addr) {
      DEBUGT("%.2x:%.2x:%.2x:%.2x:%.2x:%.2x", addr[5],addr[4],addr[3],addr[2],addr[1],addr[0]);
}

void bt_addr_ok_init( void ) {
	int bt_addr_buffer[BD_ADDR_LEN];
	int index, addr_nr;
	for (addr_nr = 0; addr_nr < NUM_OK_ADDR; addr_nr++) {	
		sscanf(_global_c_ok_addr[addr_nr], "%2x:%2x:%2x:%2x:%2x:%2x",&bt_addr_buffer[5], &bt_addr_buffer[4], &bt_addr_buffer[3], &bt_addr_buffer[2], &bt_addr_buffer[1], &bt_addr_buffer[0]);
		for (index = 0; index < BD_ADDR_LEN; index++) {
	                _sync_test_globals->ok_addr[addr_nr][index] = (u_char) bt_addr_buffer[index];
	    }
	}
}

short bt_addr_ok(bt_addr_t addr) {
	int addr_nr;	
	for (addr_nr = 0; addr_nr < NUM_OK_ADDR; addr_nr++) {	
		if(BD_ADDR_CMP(addr, _sync_test_globals->ok_addr[addr_nr]) == 1) {
			return 1;
		}		
	}
	return -1;
}

void send_auto_pkt(u_char nrdata) {

	// Prepare paket
	int res_send;
  
    //acl-pkt
    bt_acl_com_pkt_t _acl_com_pkt;
	//cast payload to acl_com
    u_char *acl_com_data = bt_acl_com_get_data_pointer(&_acl_com_pkt);

  
    int i;  
    for (i = 0; i < nrdata; i++) {
	   acl_com_data[i] = i; 		// first Byte must be 0 for Answerthread2
	}   
			
	// Choose a BTnode to send a packet to. (Choose only known BTnode)
	
	// Find all devices. Find at least 1 device.
	int nr_inq_dev;
	do {
		if (_sync_test_globals->send_freq == 0) {
			NutThreadSetPriority(255);				
		}
		#if AUTO_COMMUNICATE_DEBUG == 1
		DEBUGT("Inquiry start.\n");
		#endif
		btn_led_clear_pattern_queue();
		btn_led_add_pattern(BTN_LED_PATTERN_OFF_ON_OFF, 0x08,1, 2);
		nr_inq_dev = bt_hci_inquiry(stack, BT_HCI_SYNC, 0x03, AUTO_INQ_NUM, _sync_test_globals->inqres);
		#if AUTO_COMMUNICATE_DEBUG == 1		
		DEBUGT("Found %i devices.\n", nr_inq_dev);
		#endif
	} while (nr_inq_dev < 1);
	
	// filter out unknown devices
	u_char ok_idx[NUM_OK_ADDR]; // stores the index of known btnodes
	u_char idx_pos = 0;
	for(i = 0; i < nr_inq_dev; i++) {
		#if AUTO_COMMUNICATE_DEBUG == 1	
		DEBUGT("[%i]: ", i);	
		print_bt_addr(_sync_test_globals->inqres[i].bdaddr);
		#endif
		if(bt_addr_ok(_sync_test_globals->inqres[i].bdaddr) == 1) {
			ok_idx[idx_pos++] = i;
			#if AUTO_COMMUNICATE_DEBUG == 1
			DEBUGT(" OK\n");
			#endif
		}
		#if AUTO_COMMUNICATE_DEBUG == 1
		else {
			DEBUGT("\n");	
		}
		#endif
	}

	if(idx_pos == 0) {
		#if AUTO_COMMUNICATE_DEBUG == 1		
		DEBUGT("No known devices found!\n");
		#endif
    	//btn_led_clear_pattern_queue();
	    //btn_led_add_pattern(BTN_LED_PATTERN_DOWN, 0,1, 1);				
		return;	
	}
	
	#if AUTO_COMMUNICATE_DEBUG == 1		
	DEBUGT("%i known devices found!\n", idx_pos);
	#endif	
	btn_led_clear_pattern_queue();
	btn_led_add_pattern(BTN_LED_PATTERN_ON, (u_char) idx_pos, 4, 1);
	
	// Choose a device among known btnodes
	// (only allow known devices to avoid disturbance with other neighbour BTnodes of other semester thesis)
	// TODO: maybe removed again later.
	u_char r_index;
	r_index = ok_idx[RT_NOW % idx_pos];
	#if AUTO_COMMUNICATE_DEBUG == 1
	printf("Selected device: %i.\n", r_index);
	#endif
	
	// Is there already a connection?
	long res_handle;
	res_handle = bt_hci_get_con_handle(stack, _sync_test_globals->inqres[r_index].bdaddr);
	
	if (res_handle == BT_ERR_NO_CON) { // There is no connection yet
		// Connect to the device
		#if AUTO_COMMUNICATE_DEBUG == 1		
		DEBUGT("Connecting to device %i. With Address: ", r_index);
		print_bt_addr(_sync_test_globals->inqres[r_index].bdaddr);
		DEBUGT(".\n");
		#endif
		res_handle = bt_hci_create_connection(stack, BT_HCI_SYNC, _sync_test_globals->inqres[r_index].bdaddr, BT_HCI_PACKET_TYPE_DM1 | BT_HCI_PACKET_TYPE_DH1 | BT_HCI_PACKET_TYPE_DM3, _sync_test_globals->inqres[r_index].page_scan_rep_mode, _sync_test_globals->inqres[r_index].clock_offset, BT_HCI_ROLE_SWITCH_CHANGE);
		
	    if (res_handle < 0) { //error
	       	//btn_led_clear_pattern_queue();
		    btn_led_add_pattern(BTN_LED_PATTERN_DOWN, 0,1, 1);	
    	    #if AUTO_COMMUNICATE_DEBUG == 1
    	    DEBUGT("con: error: cannot create connection. (%lx)\n",res_handle);
    	    #endif
    	    return;
	    } else {
	    	#if AUTO_COMMUNICATE_DEBUG == 1
    	    DEBUGT("new connection open with handle %ld\n", res_handle);
    	    #endif
		}
	}
	
	if(res_handle >= 0) {
        // Send packet
			#if AUTO_COMMUNICATE_DEBUG == 1
				DEBUGT("Sending packet to handle: %ld.\n", res_handle);
			#endif
		    res_send = bt_acl_com_send_packet(acl_com_stack, &_acl_com_pkt, res_handle, 4, (u_short) nrdata);
    	    btn_led_clear_pattern_queue();
    	    if(res_send == 0) {
		        btn_led_add_pattern(BTN_LED_PATTERN_UP, 0,1, 1);
		        sync_globals.bitpattern++;
		        //if bitpattern % 16 == 0 => increment bitpattern in order to avoid
		        //	zero-pattern
		        if((sync_globals.bitpattern & 0x0f) == 0){
		        	sync_globals.bitpattern++;
		        }
 				#if AUTO_COMMUNICATE_DEBUG == 1
					DEBUGT("%d Byte sent.\n", nrdata);
				#endif				
 
    	    }
    	    else {
 		        
 		        #if AUTO_COMMUNICATE_DEBUG == 1
					DEBUGT("Send error: %d.\n", res_send);
				#endif
 		        btn_led_add_pattern(BTN_LED_PATTERN_DOWN, 0,1, 1);  	    	
    	    }
	        btn_led_heartbeat(_btn_led_heatbeat_pause, (char) sync_globals.bitpattern, 1);	    	
	}
	// wait 5 seconds to allow autosenderthread2 answering with 
	// same connection handle.
	NutSleep(5000);
	bt_hci_disconnect(stack, BT_HCI_SYNC, res_handle, BT_HCI_DISCONNECT_USER_ENDED);		
}

THREAD(autosender, arg) {
	int i, rnd;
	sync_globals.bitpattern = (char) 0;
	NutThreadSetPriority(80);
//	NutSleep(255);
	srand(btn_bat_measure(10));
	for(;;) {
		if (_sync_test_globals->send_freq != 0) {
			#if AUTO_RANDOM == 0
				rnd = 0;
			#else
				rnd = rand() % (AUTO_RANDOM * 2);
				rnd -= AUTO_RANDOM;
				#if AUTO_COMMUNICATE_DEBUG == 1
					DEBUGT("Random number: %d.\n", rnd);
				#endif				
			#endif
			for(i = 0; i <  (_sync_test_globals->send_freq + rnd); i++) {
				NutSleep(250);	
			}	
			#ifdef AUTO_LEN_RANDOM
				send_auto_pkt((u_char) (rand() % AUTOSEND_LEN) + 1);
			#else
				send_auto_pkt(AUTOSEND_LEN);
			#endif
		}else{ // send_freq = 0 means autosender is off.
			NutSleep(5000);	
		}
	}
}

void _cmd_send_auto_pkt_set_freq(u_char * arg) {
	int freq;
	if(sscanf(arg, "%u", &freq) == 1){
		_sync_test_globals->send_freq = freq;
	}
	else {
		printf("Usage: sendfreq <frequency>\n");	
	}		
}

void _cmd_auto_info(u_char * arg) {
	printf("Sendfrequency: %i.\n", _sync_test_globals->send_freq);
	printf("Allowed devices:\n");
	int i;
	for(i = 0; i < NUM_OK_ADDR; i++) {
		print_bt_addr(_sync_test_globals->ok_addr[i]);
		printf("\n");
	}
}
#endif

#if defined(BP_ISA) || defined(SYNC_RTT)
 
void _cmd_print_lilist(u_char * arg){
	print_neighbour_list();
	printf("\n>>done.\n");
}

void rt_reset(void){
	sync_neighbour * tmp_ptr = NULL;
	sync_neighbour * del_ptr = sync_globals.nearest_neighbour;
	char count = 0;
	while(del_ptr != NULL && count < SYNC_MAX_NEIGHBOURS + 1){
		tmp_ptr = del_ptr->next;
		printf(">>> %d: remove '", count++);
		bt_acl_com_print_bt_addr(del_ptr->address);
		printf("'...\n");
		NutHeapFree(del_ptr);
		del_ptr = tmp_ptr;
	}
	#ifdef BP_ISA
		rt_init(stack);
	#else // not BP_ISA
		SYNC_CURRENT_BOUNDS.validity = 0;
		rt_init();
	#endif // BP_ISA
}

void _cmd_reset_rtc(u_char * arg){
	printf("*** removing neighbours...\n");
	rt_reset();
	printf("*** Real time clock reset.\n");
}
#else // not BP_ISA || SYNC_RTT
	
	void rt_reset(void){
		sync_globals.global_bounds.lower = 0;
		sync_globals.global_bounds.upper = 0;
		sync_globals.global_bounds.validity = 0;
		rt_init();
	}
	
	void _cmd_reset_rtc(u_char * arg){
		rt_reset();
		printf("*** Real time clock reset.\n");
				}

#endif // BP_ISA || SYNC_RTT

#ifdef LLDLY
void _cmd_glldly(u_char* arg) {
	_sync_test_globals->lldly_send_ll = (long long) lldly_send;
	_sync_test_globals->lldly_rcvd_ll = (long long) lldly_rcvd;
	_lltobs(&_sync_test_globals->lldly_send_ll, _sync_test_globals->lldly_tstamp);
	_lltobs(&_sync_test_globals->lldly_rcvd_ll, _sync_test_globals->lldly_tstamp + 6);
	int i;
	printf("#");
	for(i=0; i < SYNC_TSTAMP_LEN; i += 1){
		printf("%c", ((_sync_test_globals->lldly_tstamp[i] & 0x0f) | 0x50));
		printf("%c", ((_sync_test_globals->lldly_tstamp[i] >> 4) | 0x50));
	}		
}

void _cmd_plldly(u_char* arg) {
	_sync_test_globals->lldly_send_ll = (long long) lldly_send;
	_sync_test_globals->lldly_rcvd_ll = (long long) lldly_rcvd;
	_print_dec(_sync_test_globals->lldly_send_ll);
	printf(" ");
	_print_dec(_sync_test_globals->lldly_rcvd_ll);
	printf("\n");	
}
#endif

#ifdef PUT_BOUNDS
THREAD(putboundsthread, arg) {
	int i;
	NutThreadSetPriority(160);
	while(1) {
		_cmd_gbound("");
		for(i=0; i < PUT_BOUNDS_FREQ; i++) {
			NutSleep(255);	
		}	
	}
}
#endif

#ifdef STORE_BOUNDS_LOCAL
void _cmd_print_stored_bounds(u_char* arg) {
	int i;
	#if defined(BP_ISA) || defined(SYNC_RTT)
		DEBUGT(REQ_CHAR "%%neighbour list:");
		print_neighbour_list();
		DEBUGT("\n%%lower\tupper\tlupd\tvalidity\n");
	#else // not BP_ISA || SYNC_RTT
		DEBUGT("\n%%\n%%lower\tupper\tlupd\tvalidity\n");
	#endif // BP_ISA || SYNC_RTT
	for(i = 0; i < _sync_test_globals->num_stored_bounds; i++) {
		_print_dec(_sync_test_globals->stored_bounds[i].lower);
		DEBUGT("\t");
		_print_dec(_sync_test_globals->stored_bounds[i].upper);
		DEBUGT("\t%lu\t%d\n", _sync_test_globals->stored_bounds[i].last_update, _sync_test_globals->stored_bounds[i].validity);
	}
	DEBUGT("%%" ACK_CHAR "\n");
}

void _cmd_store_bounds(u_char* arg) {
	if(_sync_test_globals->num_stored_bounds < STORE_BOUNDS_LOCAL_SIZE) {
		_sync_test_globals->stored_bounds[_sync_test_globals->num_stored_bounds++] = get_updated_bounds(&SYNC_CURRENT_BOUNDS, RT_NOW);
	}
}

void _cmd_delete_stored_bounds(u_char* arg) {
	_sync_test_globals->num_stored_bounds = 0;
}
#endif

void _cmd_print_RxComplete(u_char * arg){
	
	printf("# of RxCompletes: %lu\n", sig_UART1_RECV.ir_count);
	printf("# of RxCompletes: %lu\n", sig_UART1_DATA.ir_count);
}


#ifdef EXTERNAL_INTERRUPT

static void _sync_measure_ihndlr(void * arg){
	_sync_test_globals->sync_measure_time = RT_NOW;
	_sync_test_globals->sync_measure_signaled = 1;
}

THREAD(sync_measure_thread, arg) {
	
	NutThreadSetPriority(90);
	
	for(;;){
		if(_sync_test_globals->sync_measure_signaled == (u_char) 1){
			if(_sync_test_globals->num_stored_bounds < STORE_BOUNDS_LOCAL_SIZE){
				_sync_test_globals->stored_bounds[_sync_test_globals->num_stored_bounds++] = get_updated_bounds(&SYNC_CURRENT_BOUNDS, _sync_test_globals->sync_measure_time);
			}
			_sync_test_globals->sync_measure_signaled = 0;
		}
		NutSleep(2000);
	}
}

void _cmd_pseudo_interrupt(u_char * arg){
	_sync_test_globals->sync_measure_time = RT_NOW;
	_sync_test_globals->sync_measure_signaled = 1;
}

#endif // EXTERNAL_INTERRUPT

#if defined(IS_ANCHORNODE) && defined(ANCHOR_BEAUTY)

THREAD(sync_anchor_beauty_farm, arg){
	
	NutThreadSetPriority(120);

	NutSleep(20000);

	for(;;){
		rt_get_bounds();
		NutSleep(180000);
	}
}

#endif // IS_ANCHORNODE && ANCHOR_BEAUTY


/**
 * function that allocates a struct containing most of the global variables used
 * from heap and initializes some of them.
 */
void _init_globals(void){
	_sync_test_globals = NutHeapAlloc(sizeof(_sync_test_globals_t));
	_sync_test_globals->boundmemory.validity = 1;
	#ifdef AUTO_COMMUNICATE
		_sync_test_globals->send_freq = AUTO_COM_INIT_FREQ;
		bt_addr_ok_init();	
	#endif // AUTO_COMMUNICATE
	#ifdef STORE_BOUNDS_LOCAL
		_sync_test_globals->num_stored_bounds = 0;
	#endif // STORE_BOUNDS_LOCAL
	#ifdef AUTO_ANSWER
		_sync_test_globals->answer_state = 1;
	#endif // AUTO_ANSWER

}


/**
 * main function that initializes the hardware, led, terminal, bluetooth
 * and acl communication stack and registers some predefined commands.
 * Use tab-tab to see the registered commands once the program is running.
 */
int main(void)
{
    // serial baud rate
    u_long baud = 115200;
    //u_long baud = 57600;

    // hardware init
    btn_hardware_init();
    btn_led_init(1);
    
    //funny prompt
    u_char prompt_string[20];
    
    // init app uart
    NutRegisterDevice(&APP_UART, 0, 0);
    freopen(APP_UART.dev_name, "r+", stdout);
    freopen(APP_UART.dev_name, "r+", stdin);    
    _ioctl(_fileno(stdout), UART_SETSPEED, &baud);
    

    // bluetooth module on (takes a while)
    btn_hardware_bt_on();
    
    // verbose debug of all hci information
    //_bt_hci_debug_uart = 1;
       
    // Start the stack and let the initialization begin
    stack = bt_hci_init(&BT_UART);
    acl_com_stack = bt_acl_com_init(stack, 5);
	
	//rt init
	#ifdef BP_ISA
	    rt_init(stack);
    #else
 	   rt_init();
    #endif
	
	//initialize globals
	_init_globals();
	
	
#ifdef EXTERNAL_INTERRUPT

	NutRegisterIrqHandler(&sig_SUART_RECV, _sync_measure_ihndlr, NULL);
	//disable external interrupt before chanching the input sense
	cbi(EIMSK, SUART_RXD_INTERRUPT);
	//enable interrupt on rising edge
	sbi(EICRB, ISC61);
	sbi(EICRB, ISC60);
	//enable external interrupt
	sbi(EIMSK, SUART_RXD_INTERRUPT);

#endif // EXTERNAL_INTERRUPT


	// hello world!
	printf_P(PSTR("\n\rsync-test started..\n\r"));
	#ifdef IS_ANCHORNODE
	printf_P(PSTR("this is an anchor node.\n\r"));
	#endif
    
     printf("\n\rBT_MAX_ACL_COM_PAYLOAD: %d\n\r",  BT_MAX_ACL_COM_PAYLOAD);
     printf("BATT:%d\n", btn_bat_measure(10));
    
    // give hint
    printf_P(PSTR("hit tab twice for list of commands\n\r"));
    
    //_bt_hci_debug_uart = 1;
    // terminal init
    btn_terminal_init(stdout, "[sync-test@btnode]$");
 
 	bt_hci_read_bt_addr(stack, BT_HCI_SYNC, _sync_test_globals->my_bt_addr);
 	#ifdef IS_ANCHORNODE
		switch(_sync_test_globals->my_bt_addr[0]){
			case 0x7b : sprintf_P(prompt_string, PSTR("[sync@bernie]$"));
				break;
			case 0x7c : sprintf_P(prompt_string, PSTR("[sync@chris]$"));
				break;
			case 0x7d : sprintf_P(prompt_string,  PSTR("[sync@danny]$"));
				break;
			case 0x7f : sprintf_P(prompt_string,  PSTR("[sync@freddie]$"));
				break;
			default: sprintf_P(prompt_string, PSTR("[sync@anchor]$"));
		}
	#else
		switch(_sync_test_globals->my_bt_addr[0]){
			case 0x7b : sprintf_P(prompt_string, PSTR("[sync@brunhilde]$"));
				break;
			case 0x7c : sprintf_P(prompt_string, PSTR("[sync@claudia]$"));
				break;
			case 0x7d : sprintf_P(prompt_string,  PSTR("[sync@denise]$"));
				break;
			case 0x7f : sprintf_P(prompt_string,  PSTR("[sync@florence]$"));
				break;
			default: sprintf_P(prompt_string, PSTR("[sync@chnopf]$"));
		}
 	#endif
 	btn_terminal_set_prompt(prompt_string);
    bt_cmds_init(stack);
    bt_cmds_register_cmds();
    btn_cmds_register_cmds();
    nut_cmds_register_cmds();
    acl_com_cmds_init(stack, acl_com_stack);
    acl_com_cmds_register_cmds();
    btn_terminal_register_cmd("ptime", _cmd_print_time);     
    btn_terminal_register_cmd("gbound", _cmd_gbound); 
   	btn_terminal_register_cmd("reset", _cmd_reset_rtc);
	btn_terminal_register_cmd("rx", _cmd_print_RxComplete);
	#ifndef ESSENTIALS_ONLY
	    btn_terminal_register_cmd("msync", _cmd_sync_manual);
	    btn_terminal_register_cmd("start", _cmd_start);   
		btn_terminal_register_cmd("st", _cmd_dirty_anchor);
		#ifndef BP_ISA
	 	    btn_terminal_register_cmd("sync", _cmd_pseudo_sync);      
	    #endif
	#endif // ESSENTIALS_ONLY
	
	#ifdef LLDLY
	    btn_terminal_register_cmd("glldly", _cmd_glldly);
	    btn_terminal_register_cmd("plldly", _cmd_plldly);
	#endif
	
	#if defined(BP_ISA) || defined(SYNC_RTT)
    	btn_terminal_register_cmd("plist", _cmd_print_lilist);
	#endif
	
	#ifdef AUTO_ANSWER
		btn_terminal_register_cmd("answer", _cmd_answer);      
 		NutThreadCreate("Answr", answerthread, 0, 1024);
	#endif
	
	#ifdef AUTO_ANSWER2
		NutThreadCreate("Ans2", answerthread2, 0, 1024);
	#endif
	
	#ifdef PUT_BOUNDS
		NutThreadCreate("PutB", putboundsthread, 0, 1024);
	#endif 
		   
    #ifdef AUTO_COMMUNICATE
	    btn_terminal_register_cmd("sendfreq", _cmd_send_auto_pkt_set_freq);
	    btn_terminal_register_cmd("autoinfo", _cmd_auto_info);     
		NutThreadCreate("ASndr", autosender, 0, 1024);    
    #endif
    
    #ifdef STORE_BOUNDS_LOCAL
	    btn_terminal_register_cmd("printb", _cmd_print_stored_bounds);    	
	    btn_terminal_register_cmd("storeb", _cmd_store_bounds);
  	    btn_terminal_register_cmd("delb", _cmd_delete_stored_bounds); 	    
    #endif
    
    #ifdef EXTERNAL_INTERRUPT
		btn_terminal_register_cmd("5Volts", _cmd_pseudo_interrupt);
		NutThreadCreate("ExtIr", sync_measure_thread, 0, 256);
    #endif // EXTERNAL_INTERRUPT
	
	#if defined(IS_ANCHORNODE) && defined(ANCHOR_BEAUTY)
    	NutThreadCreate("AnchBeau", sync_anchor_beauty_farm, 0, 256);
    #endif // IS_ANCHORNODE && ANCHOR_BEAUTY
    
    // FOR MEASURING PC-ANCHOR DELAY ***  	
    /*
    int ctr;
    for(ctr=0; ctr < 100; ctr++) {
    	_cmd_print_time("");
    	NutSleep(1000);	
    }
    */
    // *********************************    
   
    btn_terminal_run(BTN_TERMINAL_NOFORK, 0);

    return 0;
}


